/***************************************************************************
                          klineakconfig.cpp  -  description
                             -------------------
    begin                : Tue Apr  9 18:33:44 EDT 2002
    copyright            : (C) 2002 by Sheldon Lee Wen
    email                : tormak@rogers.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "klineakconfig.h"
#include "pcontrol.h"
#include <map>
#include <vector>
#include <kstddirs.h>

KlineakConfig::KlineakConfig(QWidget *parent, const char *name) : KlineakConfigUI(parent, name)
{
	ApplyUsed=false;
 	changeMade=false;
	selected_key=0;
 	got_def = false;
    got_conf = false;
 	changing_text_from_signal=false;
    myEAK=&p.myEAK;
    lineakd = NULL;

    /** Set up the system tray applet */
    systemTrayApp = new Klineak(this);
    connect(systemTrayApp,SIGNAL(toggleApp()),this,SLOT(slotToggleApp()));
    connect(systemTrayApp,SIGNAL(cleanExit()),this,SLOT(quit()));

    QString homedir = QDir::homeDirPath() + LINEAKDIR;
    QDir lineakdir;
    QString picsdir = QDir::homeDirPath()  + PICSDIR;

    /** Test to see if the .lineak directory exists in the users home directory */
     lineakdir.setPath(homedir);
     if (!lineakdir.exists())
      	lineakdir.mkdir(homedir);

     /** Test to see if the ~/.lineak/Pics/ directory exists in the users home directory */
     lineakdir.setPath(picsdir);
     if (!lineakdir.exists())
      	lineakdir.mkdir(picsdir);

    /** Add the resource dirs */
    resources = KGlobal::dirs();
   // Add ROOTDIR to the prefix we will have KStandardDirs search for the image
  	resources->addPrefix(ROOTDIR);
   // Add the local Pics directory to the Icon dir search path.
   	resources->addResourceDir("icon", QString(QDir::homeDirPath()) + PICSDIR);
    KGlobal::dirs()->addResourceType("app_icon", QString(QDir::homeDirPath()) + PICSDIR);
   QStringList list = resources->resourceDirs("icon");
     /** Parse the system definition file */
     /** TODO: Make a kde config file that lists all of the users def files, and load them all here. */
    got_def = parsedef();
    QString parsehomefile = QString("%1%2%3").arg(QDir::homeDirPath()).arg(LINEAKDIR).arg(DEFFILE);
    bool ret = parsedef(parsehomefile);
    /** Parse the user definition file */
    got_def = (got_def)?got_def:ret;
    if (!got_def) {
           QMessageBox::information(this, "KLineakConfig", QString(
        	"WARNING: KLineakConfig could parse your keyboard definition file!\n" ));
			quit();
    }    	
    /** Parse the users ~/.lineak/lineakd.conf file */
    if (!(got_conf = parseconf()))
    	quit();

   // Show the panel
   systemTrayApp->show();
   // Show all the data in the gui widgets.
   refreshGUIdata();
   // Test and see if there are any lineakd processes running. If there are kill them or exit.
   int pid;
//   cout << "pid of lineakd = " << PControl::lineakdRunning() << endl;
//   if ((pid = p.lineakdRunning()) !=0) {
   if ((pid = PControl::lineakdRunning()) !=0) {
		   cout << "Killing lineakd" << endl;
   			PControl::instance()->kill(pid); //, SIGKILL);
   }  	
//  cout << "starting lineakd\n";
  startLineakd();
}

KlineakConfig::~KlineakConfig() {
   delete(resources);
   delete(systemTrayApp);
}
void KlineakConfig::slotToggleApp()
{
	if(isVisible())
		hide();
	else {
		show();
		setEnabled(true);
	}
}

bool KlineakConfig::lineakDefStart()
{
    // If it's not already running.....
    if (!klineakdef->isVisible()) {
		stopLineakd();
  		/** Parse the keyboard definitions file */
   		klineakdef->show();
   		//connect klineakdef's ok/apply or close button with lineakDefStop()
   		connect(klineakdef,SIGNAL(quitting()),this,SLOT(lineakDefStop()));
   		//connect klineakdef's ok/apply button to klineakconfig parsedef slot.
   		connect(klineakdef,SIGNAL(applying(QString)),this,SLOT(reparseDef(QString)));
   		// connect klineakdef's keyboardName signal to our setKeyboard slot
   		// connect(klineakdef,SIGNAL(keyboardName(QString)),this,SLOT(setKeyboard()));
   }
	return true;
}
/** Slot activated when KlineakDef exits. */
bool KlineakConfig::lineakDefStop()
{
   klineakdef->hide();
   startLineakd();
   return true;
}
/** Slot activated by KlineakDef's apply button. It is passed the name of the file to parse. If it is not null, then parse it. */
bool KlineakConfig::reparseDef(QString parsefile){
	cout << "reparsing" << endl;
 	if (parsefile != QString::null)
    	return (parsedef(parsefile));
   return false;
}
bool KlineakConfig::parsedef(QString parsefile) {
  QFile filename;
// 	cout << "parsing deffile" << endl;
  /** If parsefile is null, then set the default to the system lineakkb.def */
  if (parsefile == QString::null)
    		parsefile = LINEAKKB_FILE;

  filename.setName(parsefile);
  if (!filename.exists()) {
    	cerr << "The filename: " << parsefile << " that you want to parse does not exist!" << endl;
  		return false;
  } else {
      /* parse .def files */
       if(!p.parsedeffile(parsefile)) {
    		return false;
       }
       if (got_conf) {
          changing_text_from_signal = true;
          /* refresh EAK types */
          refresh_kbtypes ();
          changing_text_from_signal = false;
       }
   }
   return true;
}

bool KlineakConfig::parseconf(QString parsefile) {
  QString homedir = QDir::homeDirPath();
  QString returnMsg;
  QFile filename;

// cout << "parsing conffile\n";	

  /** If parsefile is null, then set the default to the users ~/.lineak/lineakd.conf file. */
  if (parsefile == QString::null) {
    	parsefile = homedir + CONFFILE;
       filename.setName(parsefile);
       if (!filename.exists())
       	createBasic(parsefile);

  }
   /** Parse the .conf file. If there was some error, display it and return false. */
   if((returnMsg=p.parseconffile(parsefile)) != QString::null) {
       QMessageBox::critical(this, "KLineakConfig", returnMsg);
 		return false;
   }

//  	 cout << "running initEAK\n";
    /** If we parsed a definition's file */
  	if (got_def) {
       if (!p.initEAK()) {
      	       QString homedir = QDir::homeDirPath(); //getenv("HOME");
      			homedir+=LINEAKDIR;
       			homedir+=DEFFILE;
       	
               QMessageBox::information(this, "KLineakConfig", QString(
            	"WARNING: KLineakConfig could not find any keys defined for your keyboard!\n"
             	"You probably have a corrupt or incorrect keyboard definition file in either:\n"
              "%1\n"
              "or\n"
              "%2\n").arg(LINEAKKB_FILE).arg(homedir));
        		quit();
      	}
   }
   else {
     cerr << "Attepting to load the user .conf setting before we have keyboard data!" << endl;
     return false;
  }
   return true;
}
/** Update the list of available keys in the list box when a new keyboard
	  is selected from the Keyboard Type combo box */
void KlineakConfig::updateAvailableKeys(const QString &keyboard) {
    if (myEAK->EAKtype != p.longkbname[keyboard])
		select_new_keyboard(p.longkbname[keyboard]);
}
/**
void KlineakConfig::setKeyboard(const QString &keyboard) {
	p.initEAK(keyboard);
 	cbBrand // set the brand
 	cbKeyboard // set the keyboard	
  	select_new_keyboard(keyboard);



}
*/
/** Update the list of keyboard in the Keyboard Types combo box
      when the user selects a new brand */
void KlineakConfig::updateKeyboardModels(const QString &inText) {
   if (myEAK->EAKbrand != inText) {
   		// Load the appropriate models.
    	vector<QString>::iterator it=p.bmmap[cbKeyboardBrand->currentText()].begin();
      // Clear the combo box.
      cbKeyboard->clear();
    	// For each model of keyboard
    	for(int m=0;it!=p.bmmap[cbKeyboardBrand->currentText()].end();it++, m++)  {
    		cbKeyboard->insertItem(*it);
    		if(*it == myEAK->EAKmodel)
   				cbKeyboard->setCurrentItem(m);
    	}
     	// We should now have a new keyboard in the Keyboard Type box, so update the list of keys.
   		select_new_keyboard(p.longkbname[cbKeyboard->currentText()]);
   }
}
/** Update the cdrom device text box */
void KlineakConfig::updateCDRom(const QString &inText) {
  if (inText != p.cdromdev)
       p.cdromdev = inText;
}
/** Update the mixer device text box */
void KlineakConfig::updateMixer(const QString &inText) {
  if (inText != p.mixdev)
        p.mixdev = inText;
}

// Update the keylist mappings when a radio action is selected
void KlineakConfig::updateKeyList(int index) {
  if (!radioAction->isOn())
        return;
  change_selectedkey_command(p.specialacts[index].action);
  changeMade=true;
}
// Update the keylist mappings when the text changes.
void KlineakConfig::updateKeyList(const QString& inText) {
  if (changing_text_from_signal)
        return;
  change_selectedkey_command(txtCommand->text());
    changeMade=true;
}

// When an item is selected in the keylist
void KlineakConfig::onKeyListSelect(QListBoxItem *box) {
  int index = lstAvailableKeys->currentItem();
  if (index != -1 ) {
  	changing_text_from_signal = true;
    refresh_keycommand(index);
    changing_text_from_signal = false;
    selected_key = lstAvailableKeys->currentItem();
  }
}
// When you click the Apply button
bool KlineakConfig::onApplyButtonClicked() {
  /* first time apply? -> backup conf file */
  if (!ApplyUsed) {
    QString returnMsg;
    ApplyUsed = true;
    // If there was some error message, display it.
  	if((returnMsg = p.backup_conffile()) != QString::null)
       QMessageBox::information(this,"KlineakConfig",returnMsg);
  }
  /* save the changes to a new conf file */
  if (!p.save_conffile()) {
  	QMessageBox::information(this,"KlineakConfig","ERROR: The Lineak configuration could not be saved.");	
   return false;
  }
  hide();
  /* rehash lineakd */
  hupLineakd();
  return true;
}
/** Hup the lineakd daemon.
		Return true if we hupped the daemon.
  		Return false if the daemon was not hupped */
bool KlineakConfig::hupLineakd() {
   /** Hup lineakd. If successful return true, else return false */
   if(lineakd != NULL && lineakd->kill(SIGHUP)) {
     	cerr << "HUP'd lineakd\n";
  		return true;
   }
   else {
     cerr << "Could not HUP lineakd\n";
     startLineakd();
     return false;
   }

/** lineakd has a bug. If you rehash it it becomes unresponsive. For now,
	just stop and start it */
//    int oldpid = lineakd.pid();
//    int maxrecurse = 10;
//    while (oldpid == lineakd.pid() && maxrecurse != 0)  {
//		stopLineakd();
//		startLineakd();
//		sleep(3);
//		maxrecurse--;
//	}
//		stopLineakd();
//		startLineakd();
	return (true);

}
/** Start the lineakd daemon.
		If lineakd is running standalone, kill it, and start under the control of klineakconfig.
		Return true if lineakd has been started by klineakconfig or if we started lineakd.
    	Return false if lineakd could not be started */
bool KlineakConfig::startLineakd() {
      /* If lineakd is not running */   		
    if (lineakd != NULL)
    	stopLineakd();
    	
    lineakd = new KProcess();
    lineakd->clearArguments();
  	*lineakd << "lineakd";
 	lineakd->start(KProcess::DontCare);
 	if (lineakd->isRunning()) {
//		cout << "Starting lineakd, pid is: " << lineakd->pid() << endl;
		/** At some point where I figure out how to get root priv's renice to -10 */
//		PControl pc;
//		pc.renice(lineakd->pid());
 		return true;
   }
 	else {
           QMessageBox::information(this, "KLineakConfig", QString(
        	"WARNING: KLineakConfig could not start the easy access keyboard driver!\n"));
 			cerr << "Failed to start lineakd" << endl;
  		return false;
 	}
   return false;
}
/** Stop the lineakd daemon. If we killed it return true. Else if it could not be killed
		return false */
bool KlineakConfig::stopLineakd() {
  	bool killcode=true;
//	cout << "Stopping lineakd" << endl;
    killcode = lineakd->kill();
    lineakd->clearArguments();
    p.remove_pid_file();
    delete(lineakd);
    lineakd = NULL;
 	return (killcode);
}
// When the klineak quit button is clicked
void KlineakConfig::quit() {
//  int choice = KLPrefs::instance()->mLineakAtExit;
  if (ApplyUsed) {
     hupLineakd();
  }
  else {
    	if(changeMade) {
          switch( QMessageBox::warning( this, "KLineak",
                  "Do you want to save your changes?\n",
                  "Apply",
                  "Quit", 0, 0, 1 ) )  {
              case 0: // The user clicked the Apply button or pressed Enter
                  onApplyButtonClicked(); // Apply
 				  void showPreferences();
                  break;
              case 1: // The user clicked the Quit or pressed Escape
    				p.restore_conffile();
                  break;
          }
       }
  }
	  if (lineakd != NULL && lineakd->isRunning()) {
          switch( KMessageBox::questionYesNo( this,  i18n(     //parent
                  "The lineakd daemon is still running.\n"
                  "Do you want to continue running lineakd in the background?\n"
                  "(If not, you will be unable to use your easy access keys.)\n\n"), //text
                  i18n("Leave lineakd running?"),  //caption
                  i18n("Run in background"),  //yes button
                  i18n("Stop lineakd"), //no button
                  "Lineakd At Exit"  ) ) {  // name of don't show again
              case KMessageBox::Yes : // The user clicked the "Run in background" button or pressed Enter
                  	lineakd->detach();
                  break;
              case KMessageBox::No : // The user clicked the Quit or pressed Escape
					 stopLineakd();
                  break;
          }
  	}

  KLPrefs::instance()->writeConfig();
  p.cleanexit(1);
}
/* refresh *ALL* the data on the GUI (this is called on startup) */
bool  KlineakConfig::refreshGUIdata() {
  changing_text_from_signal = true;
  /* refresh EAK types */
  refresh_kbtypes ();
  /* refresh CD-ROM entry */
  refresh_cdromdev ();
  /* refresh keyboard image */
  refresh_kbimage ();
  /* refresh key list */
  refresh_keylist ();
  /* refresh command/action */
  refresh_specials ();
  refresh_keycommand (0);
  changing_text_from_signal = false;

  return (true);
}

/* refresh EAKeyboard types */
bool  KlineakConfig::refresh_kbtypes() {

    map<QString,vector<QString> >::iterator it2 = p.bmmap.begin();
    cbKeyboardBrand->clear();
    // Load the brands of Keyboards.
 	for(int b=0;it2!=p.bmmap.end(); it2++, b++) {
    	cbKeyboardBrand->insertItem(it2->first);
       if (it2->first == myEAK->EAKbrand)
       	cbKeyboardBrand->setCurrentItem(b);
    }
    cbKeyboard->clear();
    // Load the appropriate models.
    vector<QString>::iterator it=p.bmmap[cbKeyboardBrand->currentText()].begin();
    // For each model of keyboard
    for(int m=0;it!=p.bmmap[cbKeyboardBrand->currentText()].end();it++, m++)  {
    	cbKeyboard->insertItem(*it);
    	if(*it == myEAK->EAKmodel)
   			cbKeyboard->setCurrentItem(m);
    }

  	return (true);
}
/* only needed at startup */
bool  KlineakConfig::refresh_cdromdev() {
  txtCDRom->setText(p.cdromdev);
  return (true);
}

/* refresh keyboard image */
bool  KlineakConfig::refresh_kbimage () {
	const QString pixmapname=myEAK->EAKtype;
   QPixmap tmp;
   QString localdir = QString(QDir::homeDirPath()) + PICSDIR;
   // If we don't get a valid picture for our keyboard from the system directory
	if ( load.iconPath(pixmapname,KIcon::User, true) == QString::null) {
		// Look in the users ~/.lineak/Pics/ directory for any file matching the keyboard name.
      QDir imgdir( localdir );
   		imgdir.setFilter( QDir::Files );
   		// Go through the list of files
       const QFileInfoList* files = imgdir.entryInfoList();
       QFileInfoListIterator it( *files );
       QFileInfo * fi;
       QString picname;
       // Go through the ~/.lineak/Pics directory and look at the picture files.
       while ( (fi = it.current()) != 0 ) {
       	if ( fi->fileName() == "." || fi->fileName() == ".." ) {
          		++it;
   				continue;
   			}
   			if ( fi->isFile() && fi->isReadable()) {
       		picname = fi->fileName();
           	if (picname.contains(pixmapname)) {
            		cout << "Found it! file = " << fi->filePath() << endl;
              	picname = fi->filePath();
           		break;
              }
       	}
   			++it;
       }  // end while
		if ( (tmp = QPixmap(picname)).isNull() )
   			tmp = load.loadIcon("noimage", KIcon::User);
    }
    else
    	tmp = load.loadIcon(pixmapname,KIcon::User);
//    cout << "Iconpath = " <<  load.iconPath(pixmapname,KIcon::User, true) << endl;
//  	pixPicture->setPixmap(load.loadIcon(pixmapname,KIcon::User));
	pixPicture->setPixmap(tmp);
  	return (true);
}

/* refresh the keylist when the user selected some other keyboard */
bool  KlineakConfig::refresh_keylist() {
  QString tmp;
  EAkey *thiskey=myEAK->EAKeylist;
  lstAvailableKeys->clear();
  for (; thiskey!=NULL; thiskey=thiskey->next) {
	tmp = thiskey->EAkeyname;
	lstAvailableKeys->insertItem(tmp);
  }
  lstAvailableKeys->setSelected(0,true);
  selected_key = 0;
  return (true);
}

/* refresh the command bound to a key when it's selected */
bool  KlineakConfig::refresh_keycommand(int keyindex) {
  QString tmp;
  EAkey *thiskey=myEAK->EAKeylist;
  changing_text_from_signal = true;
  int i=0;
  for (i=0; i!=keyindex && thiskey!=NULL; i++, thiskey=thiskey->next);
  if (thiskey==NULL) {
	cerr << "Invalid key index: " << keyindex << endl;
	return (false);
  }
  if (is_special_action(thiskey->EAcommand)) { /* special action */
	radioRun->setChecked(false);
	radioAction->setChecked(true);
	txtCommand->setText("");
	txtCommand->setEnabled(false);
	for (i=0; p.specialacts[i].action != thiskey->EAcommand && i<NR_SPECIALS; i++);
	cbAction->setCurrentText(p.specialacts[i].longname);
  } else {                                                        /* system command */
	radioAction->setChecked(false);
	radioRun->setChecked(true);
	txtCommand->setText(thiskey->EAcommand);
	txtCommand->setReadOnly(false);
  }
  changing_text_from_signal = false;
  return (true);
}

/* refill special action menu (only needed on startup) */
bool  KlineakConfig::refresh_specials () {
  for (int i=0; i<NR_SPECIALS; i++)
  	cbAction->insertItem(p.specialacts[i].longname);
  return (true);
}

/* is a command a special action? */
bool  KlineakConfig::is_special_action (const QString &command) {
  if (command.compare(EAK_EJECT)==0 ||
	  command.compare(EAK_VOLUP)==0 ||
      command.compare(EAK_VOLDOWN)==0 ||
      command.compare(EAK_MUTE)==0 ||
      command.compare(EAK_SLEEP)==0)
    return (true);
  else
	return (false);
}

/* select a new keyboard */
void  KlineakConfig::select_new_keyboard(const QString &ndata) {
  p.kbtype = ndata;
  p.initEAK();
  changing_text_from_signal = true;
  refresh_keylist();
  refresh_keycommand(0); /* refresh_keylist selects 1st key, hence index 0 */
  refresh_kbimage();
  changing_text_from_signal = false;
  selected_key = 0;
}

/* change the key command data to the users input */
void  KlineakConfig::change_key_command(EAkey *thiskey, const QString &entrytext) {
  thiskey->EAcommand = entrytext;
}

/* wrapper for change_key_command, lookups the selected key */
void  KlineakConfig::change_selectedkey_command(const QString &entrytext) {
  EAkey *thiskey;
  int i;
  for (i=0, thiskey=myEAK->EAKeylist; i!=selected_key && thiskey!=NULL; i++, thiskey=thiskey->next);
  if (thiskey==NULL) {
    cerr << "Invalid selected key index: " << selected_key << endl;
    return;
  }
  if (thiskey->EAcommand != entrytext)
	change_key_command(thiskey, entrytext);
}
/** Present the user with a choice of supported keyboards and get the keyboard type back as a return value. */
void KlineakConfig::createBasic(const QString &filename){
//  QString brand = p.bmmap.begin()->first;
  /** Get the first keyboard parsed from the .def file */
  QString kbtype = p.longkbname[*(p.bmmap[p.bmmap.begin()->first].begin())];

  // Write a mini config file.
  p.conffilename.close();
  // Set the filename
  p.conffilename.setName(filename);
  if (!p.conffilename.open(IO_WriteOnly)) {
     QMessageBox::critical(this, "KLineakConfig",
  	"Error opening configuration file for writing!\n");
	cerr << "Error opening configuration file " << p.conffilename.name().latin1() << " for write." << endl;
 	p.conffilename.close();
    p.cleanexit(false);
  }
  QTextStream t(&p.conffilename);
  t << QString("%1# LinEAK Configuration \n# (generated by lineakconfig)\n\n# Global settings\n"
               "KeyboardType\t\t= %2\n"
               "\n# Specific keys of your keyboard\n").arg(CONF_HEADER).arg(kbtype);

  t << QString("\n# end lineakd.conf\n");
  p.conffilename.close();
}
/** This member function set's KlineakConfig's definition widget. */
void KlineakConfig::setLineakDef(KlineakDef *def){
	klineakdef = def;	
}
