/***************************************************************************
                          klineakconfig.h  -  description
                             -------------------
    begin                : Tue Apr  9 18:33:44 EDT 2002
    copyright            : (C) 2002 by Sheldon Lee Wen
    email                : tormak@rogers.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KLINEAKCONFIG_H
#define KLINEAKCONFIG_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qwidget.h>
#include <klineakconfigui.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qmessagebox.h>
#include <kmessagebox.h>
#include <qstring.h>
#include <kstddirs.h>
#include <kprocess.h>
#include <kiconloader.h>
#include <qfileinfo.h>
#include <kurl.h>

#include "lineakparser.h"
#include "klineak.h"
#include "klineakdef.h"
#include "klprefs.h"

class KlineakDef;
class LineakParser;
class Klineak;
class PControl;

/** KlineakConfig is the base class of the project */
class KlineakConfig : public KlineakConfigUI
{
  Q_OBJECT 
  public:
    /** construtor */
    KlineakConfig(QWidget* parent=0, const char *name=0);
    /** destructor */
    ~KlineakConfig();

   private:
		bool ApplyUsed;
  		bool changeMade;
    	bool got_def;
     	bool got_conf;
    	bool changing_text_from_signal;
     	int selected_key;
       LineakParser p;
      	Klineak *systemTrayApp;
       KStandardDirs *resources;
       KIconLoader load;
       EAKeyboard *myEAK;
       KProcess *lineakd;
       KlineakDef  *klineakdef;
       
   public slots:
       virtual void updateKeyList(int index);
       virtual void updateKeyList(const QString& inText);
       virtual void updateKeyboardModels(const QString& inText);
       virtual void onKeyListSelect(QListBoxItem *box);
       virtual void updateAvailableKeys(const QString &keyboard);
       virtual void updateCDRom(const QString& inText);
       virtual void updateMixer(const QString& inText);
       virtual void quit();
       virtual bool onApplyButtonClicked();
       virtual void slotToggleApp();
       virtual bool hupLineakd();
       virtual bool startLineakd();
       virtual bool stopLineakd();
       virtual bool lineakDefStart();
       virtual bool lineakDefStop();
       virtual bool parsedef(QString parsefile=QString::null);
       virtual bool parseconf(QString parsefile=QString::null);
  		/** This member function set's KlineakConfig's definition widget. */
  		virtual void setLineakDef(KlineakDef *def);
  		/** Slot activated by KlineakDef's apply button. It is passed the name of the file to parse. If it is not null, then parse it. */
  		virtual bool reparseDef(QString parsefile = QString::null);

   private:
		/* refresh *ALL* the data on the GUI (this is called on startup) */
  		bool refreshGUIdata();
    	/* refresh EAK types */
     	bool refresh_kbtypes();
       /* refresh CD-ROM entry */
       bool refresh_cdromdev();
       /* refresh keyboard image */
       bool refresh_kbimage();
       /* refresh key list */
       bool refresh_keylist();
       /* refresh command/action */
       bool refresh_keycommand(int i);
       /* refresh the special commands list */
       bool refresh_specials();
       /** Is this command a special action? */
       bool is_special_action (const QString &command);
       /* select a new keyboard */
       void select_new_keyboard(const QString &ndata);
       /* change the key command data to the users input */
       void change_key_command(EAkey *thiskey, const QString &entrytext);
       /* wrapper for change_key_command, lookups the selected key */
       void change_selectedkey_command(const QString &entrytext);
  		/** Present the user with a choice of supported keyboards and get the keyboard type back as a return value. */
  		void createBasic(const QString &filename);

};

#endif
