/***************************************************************************
                          lineakparser.cpp  -  description
                             -------------------
    begin                : Sat May 25 2002
    copyright            : (C) 2002 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lineakparser.h"

LineakParser::LineakParser() {
	specialacts[0] = specialact(EAK_EJECT, "Eject CD-ROM");
	specialacts[1] = specialact(EAK_VOLUP, "Increase volume");
 	specialacts[2] = specialact(EAK_VOLDOWN, "Decrease volume");
  	specialacts[3] = specialact(EAK_MUTE, "Mute volume");
   	specialacts[4] = specialact(EAK_SLEEP, "Sleep");
    kbtype=QString("");
 	cdromdev=QString("");
  	mixdev=QString("");
	conffilename.setName("");
 	deffilename.setName("");
  	backupfile.setName("");
	nrkeyboards=0;
	confdata = NULL;
	defdata = NULL;
 	userdefdata = NULL;
}
LineakParser::~LineakParser(){
}

/* parse .conf file */
QString LineakParser::parseconffile(const QString &inFile) {
  QString homedir = "";
  QString filename;

    conffilename.setName(inFile);
    	
  /* set (globally) full path of config file ($HOME/.lineak/CONFIGFILE) */
  if (conffilename.name() == "" || conffilename.name() == QString::null) {
    homedir = QDir::homeDirPath();
    conffilename.setName(homedir + CONFFILE);
    if (!conffilename.exists()) {
      	conffilename.close();

	    cerr << "\nNOTE: configuration file not found,\nplease run \"lineakd -c TYPE\" to create one for your keyboard.\n";
	    return ("Configuration file not found,\nplease run \"lineakd -c TYPE\" to create one for your keyboard.\n");
    }
  }
  /* now that we're sure it exists, parse the contents */
  if ( (confdata = dcfp_parse_file(conffilename.name().latin1())) == NULL ) {
        /* error parsing file */
        cerr << "FATAL ERROR occurred while reading configuration data!" << endl;
        return ("FATAL ERROR occurred while reading configuration data!");
  }
  /* store keyboard type globally */
  if ((kbtype = (char *)dcfp_get_global_var(confdata, "KeyboardType")) == "") {
        cerr << "*** Error in config file: KeyboardType not found!" << endl;
        return ("Error in config file: KeyboardType not found!");
  }
  /* store cdrom device globally, defaults to /dev/cdrom */
  if ((cdromdev = (char *)dcfp_get_global_var(confdata, "CdromDevice")) == "") {
        cerr << "*** CdromDevice not found in config file, setting default to /dev/cdrom..." << endl;
        cdromdev = "/dev/cdrom";
  }
  /* store cdrom device globally, defaults to /dev/mixer */
  if ((mixdev = (char *)dcfp_get_global_var(confdata, "MixerDevice")) == "") {
        cerr << "*** MixerDevice not found in config file, setting default to /dev/mixer..." << endl;
        mixdev = "/dev/mixer";
  }
  	conffilename.close();
   return (QString::null);
}

/* parse keyboard definition file */
bool LineakParser::parsedeffile(const QString &inFile) {
  dcfp_section_struct *sectptr=NULL;
  dcfp_var_struct *varptr=NULL;
  const dcfp_data_struct *tmpdefdata=NULL;

  /** If the inFile is null, that means that we have nothing to parse.
  		 If inFile == LINEAKKB_FILE then check to make sure that defdata == NULL b/c
     		we only want to parse this once.
       if ifFile != LINEAKKB_FILE then append the data to the defdata structure.
       */

   /** If the request is to parse the system deffile and we have already done that then return */
   if (inFile == LINEAKKB_FILE && defdata != NULL)
   		return true;

  	deffilename.setName(inFile);
   /** First check to see if the file exists. If it does, parse it.*/
   if (deffilename.exists()) {
     	if ( (tmpdefdata = dcfp_parse_file(deffilename.name().latin1())) == NULL)
        	cerr << "ERROR occurred while reading definition data from" << deffilename.name() << endl;
   }
  /* If this is the first time parsing, and we could not parse the file. */
  if  (defdata == NULL && tmpdefdata == NULL) {
  	/* error parsing file */
    cerr << "ERROR occurred. No keyboard definition data was added!" << endl;
    return false;
  }
  /** If the system .def file has been parsed, then this is the user defined data */
  if  (defdata != NULL && tmpdefdata != NULL) {
		if ( userdefdata != NULL) {
			dcfp_section_struct *sectptr=NULL;
			dcfp_var_struct *varptr=NULL;
       	  	// Remove the old user data.
       	  	varptr = defdata->globalvars;
           	if (varptr==NULL) {
            	varptr = tmpdefdata->globalvars;
           	} else {
            	while (varptr->next != userdefdata->globalvars) varptr = varptr->next;
             	varptr->next = NULL;
           	}
       		sectptr = defdata->sections;
           	if (sectptr == NULL) {
            	sectptr = tmpdefdata->sections;
           	} else {
            	while (sectptr->next != userdefdata->sections) sectptr = sectptr->next;
             	sectptr->next = NULL;
           	}
    	}
     	// Destroy the old user data.
      dcfp_destroy_data_struct((dcfp_data_struct *)userdefdata);
  		userdefdata = tmpdefdata;  		
  }

  /** If this was the first time this is called, set defdata to point to tmpdefdata */
  if  (defdata == NULL && tmpdefdata != NULL)
  		defdata = tmpdefdata;
  /** Else, append the data */
  else {
     /** Append the user data */
      if (tmpdefdata != NULL) { //append user def data
       	varptr = defdata->globalvars;
           if (varptr==NULL) {
             varptr = tmpdefdata->globalvars;
           } else {
             while (varptr->next != NULL) varptr = varptr->next;
             varptr->next = tmpdefdata->globalvars;
           }
       	sectptr = defdata->sections;
           if (sectptr == NULL) {
             sectptr = tmpdefdata->sections;
           } else {
             while (sectptr->next != NULL) sectptr = sectptr->next;
             sectptr->next = tmpdefdata->sections;
           }
     }
  }

   dcfp_section_struct *sections=defdata->sections;
   dcfp_var_struct *globs;
   /** Clear the existing maps and add the complete data */
   longkbname.clear();
   bmmap.clear();
   /* Fill the longkbname map and get a list of brandnames */
   while (sections !=NULL) {
     QString kbtype, kbname, brand, model;
      kbname = sections->name;
      globs=sections->globalvars;
      while (globs!=NULL) {
      		if(QString(globs->key) == "brandname")
          		brand =(char*)globs->value;
          if(QString(globs->key) == "modelname") {
            	model = (char*)(globs->value);
      			longkbname[model] = kbname;
         		bmmap[brand].push_back(model);
          }
           globs=globs->next;
      }
      sections=sections->next;
   }
	deffilename.close();

  return true;
}

/* init the EAKeyboard */
bool LineakParser::initEAK (const QString &inKBType) {
  const dcfp_section_struct *kbsect=NULL, *keysect=NULL;
  dcfp_var_struct *keyptr;
  EAkey *tmp = NULL;
  char * comm;
  int keycount = 0;

  // If inKBType is not empty, then set kbtype to be the parameter passed in.
  if (inKBType != QString::null )
  		kbtype = inKBType;

  // Goto the section for the users keyboard type eg.  LTCDP
  kbsect = dcfp_get_global_section_by_name(defdata, kbtype.latin1());
  myEAK.EAKtype = kbtype;
  // Get the brandname and modelname for our keyboard.
  myEAK.EAKbrand = (char *)dcfp_get_var_from_section(kbsect, "brandname");
  myEAK.EAKmodel = (char *)dcfp_get_var_from_section(kbsect, "modelname");
  // Get the KEYS section for our keyboard
  if ((keysect = dcfp_get_subsection_by_name(kbsect, "KEYS")) == NULL) {
        cerr << "*** Error: no keys found in your keyboard section!" << endl;
        return false;
  }

  keyptr = keysect->globalvars;
  if (keyptr != NULL) {
        myEAK.EAKeylist = tmp = new EAkey();
        while(keyptr!=NULL) {
	      keycount++;
          tmp->EAkeyname = QString(keyptr->key);
          tmp->EAkeycode = *(int *)(keyptr->value);
          tmp->EAkeysym = 0x1008FF01 + keycount;
          tmp->EAcommand = "";
          if ( ( comm = (char *)dcfp_get_global_var(confdata, tmp->EAkeyname)) ==NULL)
                cerr << "*** ERROR: Command for key '" << tmp->EAkeyname << "' does not exist in the configuration file!" << endl;
          else
          		tmp->EAcommand = QString(comm);
          if (keyptr->next != NULL) {
                tmp->next = new EAkey();
                tmp = tmp->next;
          } else {
                tmp->next = NULL;
          }
          keyptr = keyptr->next;
        }
  } else {
        myEAK.EAKeylist = NULL;
  }
  return true;
}

/* destroy the EAkeys linked list */
void LineakParser::cleanKeylist (void) {
  EAkey *tmp = NULL, *tmp2 = NULL;
  tmp = myEAK.EAKeylist;
  while (tmp!=NULL) {
        tmp2 = tmp->next;
        tmp->EAkeyname = "";
        tmp->EAcommand = "";
        delete(tmp);
        tmp = tmp2;
  }
}

/* cleanup() only used when really bailing out :) */
void LineakParser::cleanConfdata (void) {
  /* free data structures */
  if (confdata != NULL)
  	dcfp_destroy_data_struct((dcfp_data_struct *)confdata);
}

/* cleanup() only used when really bailing out :) */
void LineakParser::cleanDefdata (void) {
  /** We don't clean userdefdata b/c we have appended it onto the
  		defdata structure. When we free defdata, it will free the userdefdata
    	as well */

  /* free data structures */
  if (defdata != NULL) {
    dcfp_destroy_data_struct((dcfp_data_struct *)defdata);
  }
}

/* cleanup */
void LineakParser::cleanexit(int status) {
  cleanConfdata();
  cleanDefdata();
  /* and clean up that linked list */
  cleanKeylist();
  exit(status);
}

/* backup the conf file so we can cancel the changes */
QString LineakParser::backup_conffile (void) {
	KURL source, dest;
  QString name;
  QString returnMsg;
   cout << "Backup conffile\n";
   cout << conffilename.name() << " to " << conffilename.name() + ".backup" << endl;
  source.setPath(conffilename.name());
  dest.setPath(conffilename.name() + ".backup");
  KIO::file_copy(source,dest, 0755, true, true, false);
  
  return (QString::null);
}

/* restore the conf file from backup */
bool LineakParser::restore_conffile (void) {

	KURL source, dest;
  	source.setPath(conffilename.name() + ".backup");
  	dest.setPath(conffilename.name());
  	KIO::file_copy(source,dest, 0755, true, true, false);

  return true;
}

/* save the settings to the conf file */
bool LineakParser::save_conffile (void) {
  EAkey *key;
  conffilename.close();
  cout << "Saving conffile\n";
  if (!conffilename.open(IO_WriteOnly)) {
	cerr << "Error opening configuration file " << conffilename.name().latin1() << " for write.\n" << endl;
 	conffilename.close();
	return false;
  }
  QTextStream t(&conffilename);
  t << QString("%1# LinEAK Configuration for %2\n# (generated by lineakconfig)\n\n# Global settings\n"
               "KeyboardType\t\t= %3\n"
               "CdromDevice\t\t= %4\n"
               "MixerDevice\t\t= %5\n"
               "\n# Specific keys of your keyboard\n").arg(CONF_HEADER).arg(myEAK.EAKmodel) \
               .arg(kbtype).arg(cdromdev).arg(mixdev);

  key = myEAK.EAKeylist;
  while (key!=NULL) {
    t << QString("%1%2= \"%3\"\n").arg(key->EAkeyname).arg(key->EAkeyname.length()>7?"\t":"\t\t").arg(escapeQuotes(key->EAcommand));
	key = key->next;
  }
  t << QString("\n# end lineakd.conf\n");
  conffilename.close();
  return true;
}

void LineakParser::remove_pid_file(void) {
  QFile fp;
  QString home;
  home = QString(QDir::homeDirPath());
  home+=QString(PIDFILE);
  fp.setName(home);
  fp.remove();
}

void LineakParser::print() {
   dcfp_var_struct *globs = defdata->globalvars;
   dcfp_section_struct *sections=defdata->sections;
 	if (globs == NULL) cout << "globs is NULL\n";
	if (sections == NULL) cout << "sections is NULL\n";
   cout << "PRINTING GLOBAL VARS\n";
  while (globs!=NULL) {
   		cout << "type=" << globs->type << endl;
     	cout << "key=" << globs->key << endl;
       cout << "value=" << (char*)(globs->value) << endl;
       globs=globs->next;
    }
   cout << "PRINTING SECTION VARS\n";
   while (sections !=NULL) {
      cout << "Printing globalvars for section " << sections->name << endl;
      globs=sections->globalvars;
      while (globs!=NULL) {
        	cout << "type=" << globs->type << endl;
          	cout << "key=" << globs->key << endl;
           cout << "value=" << (char*)(globs->value) << endl;
           globs=globs->next;
      }
      cout << "Section header=" << sections->sections->name << endl;
      cout << endl << endl;
      sections=sections->next;
   }
}
/** Strip the escaped quotes out of a command. */
QString LineakParser::stripEscapedQuotes(QString command){
	int index=0;
	while ((index = command.find('"', index)) != -1) {
	    	if ((index - 1) >= 0 && command[index-1] == '\\') {
				command.remove(index,1);
//				index ++;
			}
			else
				index ++;
	}
	cout << "escapeQuotes returning: " << command << endl;
	return command;
}
/** Escape the quotes in a command. */
QString LineakParser::escapeQuotes(QString command){
	int index=0;
	while ((index = command.find('"', index)) != -1) {
	    	if ((index - 1) >= 0 && command[index-1] != '\\') {
				command.insert(index,'\\');
				index +=2;
			}
			else
				index ++;
	}
	cout << "escapeQuotes returning: " << command << endl;
	return command;
}
