/***************************************************************************
                          pcontrol.cpp  -  description
                             -------------------
    begin                : Fri Jun 28 2002
    copyright            : (C) 2002 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <kdebug.h>

#include "pcontrol.h"
#include <kstaticdeleter.h>

PControl *PControl::mInstance = 0;
static KStaticDeleter<PControl> insd;

PControl::PControl(QWidget *parent, const char *name ) : QWidget(parent,name) {
}
PControl::~PControl(){
  kdDebug() << "KLPrefs::~KLPrefs()" << endl;
  if (mInstance == this)
      mInstance = insd.setObject(0);
}
PControl *PControl::instance() {
  if (!mInstance)
      mInstance = insd.setObject(new PControl());

  return mInstance;
}

/** Returns the PID of process "process" It will only return one pid. */
int PControl::processPID(const QString process) {
   int uid = getuid();
   QDir procdir( "/proc" ), wrkdir;
   // Attempt to mount /proc
	//KIO::mount(false,"proc","none","/proc");
    // Set the directory filters
	procdir.setFilter( QDir::Dirs );
	procdir.setNameFilter( "[0-9]*"); //\\d*$" ); // only look at directories that are numbers
	// Go through the list of directories
    const QFileInfoList* files = procdir.entryInfoList();
    QFileInfoListIterator it( *files );
    QFileInfo * fi;
    QFile status;
    QString cmdline;
    // Go through all of the directories and look at the status files.
    while ( (fi = it.current()) != 0 ) {
       if ( fi->fileName() == "." || fi->fileName() == ".." ) {
       	++it;
			continue;
		}
		if ( fi->isDir() && fi->isReadable() && (fi->ownerId() == (unsigned int)uid) ) {
				wrkdir.setPath(fi->filePath());
				status.setName(wrkdir.path() + "/status");
				if (!status.open(IO_ReadOnly)) {
					cerr << "Error opening file " << status.name() << " for reading.\n" << endl;
 					status.close();
  				}
      			QTextStream dat(&status);
              // Throw away the Name: part
           	dat >> cmdline;
         		dat >> cmdline;
           	if (cmdline.contains(process)) {
					status.close();
            		cout << process << " PROCESS FOUND. pid = " << wrkdir.dirName().toInt()<< endl;
              	// Return the pid if it's not 0 and not our current process.
              	if (wrkdir.dirName().toInt() != 0 && wrkdir.dirName().toInt() != getpid())
               		return (wrkdir.dirName().toInt());
              }
				status.close();
       }
		++it;
    }
    return 0;
}
/** Renice process pid. */
void PControl::renice(int pid){
	if (setpriority(PRIO_PROCESS, pid, -10) == 0) {
		cout << "reniced " << pid << endl;
		emit niced(pid);
	}
	else
		cout << "could not renice " << pid << endl;
}
/** Returns the pid of a lineakd process. It returns 0 if one is not found. */
int PControl::lineakdRunning() {
	return processPID("lineakd");
}
/** Returns the pid of a klineakconfig process. Returns 0 if not found. Convienience method. */
int PControl::klineakconfigRunning() {
    int pid;
    if ((pid = processPID("klineakconfig")) != getpid())
		return pid;
	return 0;
}
/** Sends process PID pid., the SIGKILL signal. */
void PControl::kill(int pid){
	if (::kill(pid,SIGKILL) == 0)
		emit killed(pid);
}
